--
-- Metadata for generating data entry forms
--
-- Based on CMS form metadata code.
--
-- The core datamodel allows applications to store metadata to generate html
-- forms for use with dynamic object types.
--
-- @author Lee Denison (lee@xarg.co.uk)

---------------------------------------------
-- Core reference datamodel
---------------------------------------------

create table dtype_widgets (
  widget           varchar(100)
                   constraint dtype_widgets_pk
                   primary key
);

comment on table dtype_widgets is '
  Canonical list of all widgets defined in the system.
';

---------------------------------------------
-- Form instance datamodel
---------------------------------------------

create table dtype_layouts (
  layout_id           integer
                      constraint dtype_layout_layout_id_fk
                      references acs_objects (object_id)
                      constraint dtype_layout_layout_id_pk
                      primary key,
  type                varchar(100)
                      constraint dtype_layout_type_nn
                      not null
                      constraint dtype_layout_type_ck
                      check (type in ('form', 'element', 'object', 'attribute')),
  name                varchar(200)
                      constraint dtype_layout_name_nn
                      not null,
  pretty_name         varchar(200)
                      constraint dtype_layout_pretty_name_nn
                      not null,
  description         varchar(200),
  filename            varchar(200),
  resource_dir        varchar(200),
  constraint dtype_layout_name_type_unq unique (name, type)
);

comment on table dtype_layouts is '
  The possible ats layouts for forms and elements as well as displaying
  object types.
';

create table dtype_forms (
  form_id             integer
                      constraint dtype_form_id_pk
                      primary key
);

create table dtype_elements (
  element_id          integer
                      constraint dtype_elements_pk
                      primary key,
  name                varchar(1000)
                      constraint dtype_elements_name_nn
                      not null,
  form_id             integer
                      constraint dtype_elements_form_ref
                      references dtype_forms
                      constraint dtype_elements_form_nn
                      not null,
  parent_id           integer
                      constraint dtype_elements_parent_ref
                      references dtype_elements,
  max_child_sortkey   varbit,
  tree_sortkey        varbit,
  widget	          varchar(100)
                      constraint dtype_elements_widget_fk
                      references dtype_widgets,
  layout_id           integer
                      constraint dtype_elements_layout_ref
                      references dtype_layouts,
  custom              varchar(100),
  constraint dtype_elements_form_name_unq unique (form_id, name)
);

---------------------------------------------
-- dtypes integration datamodel
---------------------------------------------

create table dtype_form_type_map (
  form_id             integer
                      constraint dtype_form_map_form_ref
                      references dtype_forms
                      constraint dtype_form_map_form_id_nn
                      not null,
  form                varchar(1000)
                      constraint dtype_form_map_form_nn
                      not null,
  object_type         varchar(1000)
                      constraint dtype_form_map_obj_type_ref
                      references acs_object_types
                      constraint dtype_form_map_obj_type_nn
                      not null,
  constraint dtype_form_type_map_id_unq unique (form_id, object_type),
  constraint dtype_form_type_map_form_unq unique (form, object_type)
);

create table dtype_element_attribute_map (
  element_id          integer
                      constraint dtype_elm_map_elm_ref
                      references dtype_elements
                      constraint dtype_elm_map_elm_nn
                      not null,
  attribute_id        integer
                      constraint dtype_elm_attr_fk
                      references acs_attributes
                      constraint dtype_elm_attr_nn
                      not null,
  constraint dtype_elm_attr_map_unq unique (element_id, attribute_id)
);
